#ifndef ADAPTIVITREE_H
#define ADAPTIVITREE_H

#include <QtGui/QMainWindow>
#include "ui_adaptivitree.h"
#include "ui_resultDialog.h"
#include "ui_winnerDialog.h"
#include "ui_newDialog.h"

#include <QFileDialog>
#include <QDialog>
#include <QKeyEvent>
#include <QString>
#include <QStringList>


#include "xmlReader.h"
#include "xmlWriter.h"
#include <iostream>

/**
* @class AdaptiviTree
*
* @brief  The main window
*
* The main window contains the canvas where the tournament is drawn, 
* a toolbar which contains buttons for creating a new, open an existing or save the current tournament,
* a display where the team names and icons as well as the result of the current game are displayed and
* a time slider which represents the tournament progress.
*/ 

class AdaptiviTree : public QMainWindow
{
	Q_OBJECT

public:
	  
	AdaptiviTree(QWidget *parent = 0, Qt::WFlags flags = 0);
	~AdaptiviTree();
	/**
	* XML file name
	*/
	QString filename;


protected:
	void keyPressEvent(QKeyEvent* event);

private:
	/// UI for main window
	Ui::AdaptiviTree ui;
	///UI for result dialog
	Ui::ResultDialog resultUI;
	/// UI for winner dialog
	Ui::WinnerDialog winnerUI;
	/// UI for new tournament dialog
	Ui::NewDialog newUI;
	QDialog* dialogResult;
	QDialog* dialogWinner;
	QDialog* dialogNew;
	/**
	* Vector containing Team names inserted by user
	*/
	QVector<QLineEdit*> teamInput;
	/**
	* Vector containing file names of icons inserted by user
	*/
	QVector<QString> iconInput;
	/**
	* Vector containing tool buttons
	*/
	QVector<QToolButton*> toolButtons;
	/**
	* Vector containing icon labels
	*/
	QVector<QLabel*> iconLabels;
	/**
	* Actual value of the combobox in new-tournament dialog
	*/
	
	int comboBoxValue;
	/**
	* Action triggered by the slider
	*/
	int sliderAction;
	//int mouseValue;
	
	
	void init(void);
	/**
	* Updates the display  
	*/
	void updateDisplay(Game* game);
	/**
	* @brief Clears the display
	* 
	* In the beginning no game is played so the display is empty
	*/
	void clearDisplay(void);
	/**
	* Opens a dialog to edit the Game Result
	*/
	void editGameResult(Game* game);
	/**
	* Displays the winner at the end of the tournament
	*/
	void displayWinner(void);
	void setSliderStep(int value);
	/**
	* An example Tournament shown at the beginning
	*/
	void exampleTournament(void);

private slots:
	void openFileDialog(void);
	void sliderValueChanged(int value);
	void setSliderAction(int action);
	void saveFileDialog(void); 
	void newTournament(void);
	void comboBoxValueChanged(QString value);
	void iconFileDialog(void);
	//void getMouseValue(int value);
};

#endif // ADAPTIVITREE_H
