#ifndef TEAM_H
#define TEAM_H

#include <QString>
#include <QPixmap>


/**
* @class Team
*
* @brief Contains the information and handles of a Team.
*
* Contains information like name, icon, ect.
* Also contains which pick was set for this team.
*/ 
class Team
{
public:
	Team(void);
	/**
	*`Creates a new Team with name and an icon.
	*/
	Team(QString name, QString icon);

	/**
	* Creates a new Team with all information.
	*/
	Team(int id,  QString name, QString info, QString icon);
	~Team(void);

	enum Round {NO,SIXTEEN, EIGHT, QUARTER, HALF, FINAL};
	//getters
	QString getName(void);
	QPixmap getIcon(void);
	QString getInfo(void);
	QString getIconFileName(void);
	int getId(void);
	Round getPick(void);
	static Round getPickForInt(int r);
	bool hasLost(void);
	//setters
	void setName(QString name);
	void setPick(Round r);
	void setIcon(QString iconFile);
	void setIconFile(QString filename);
	void setState(bool lost);
	
private:
	static int count;
	int id;
	QString name;
	QPixmap icon;
	/// File name of the icon
	QString iconFile;
	QString info;
	Round pick;
	/// Indicates if a Team drops out of the Tournament
	bool lost;
};

#endif
