#include "tournament.h"

const int Tournament::VALID_TEAMCOUNTS[] = {4,8,16,32};

Tournament::Tournament(void) {}

Tournament::Tournament(const int teamcnt, QString title)
{
	if(isValidTeamcount(teamcnt))
	{
		this->teamcnt = teamcnt;
		this->bracketcnt = teamcnt/4;
		this->title = title;
		initTeams();
		initGames();
		initPicks();
	}
	else
		throw 1;
}

Tournament::~Tournament(void)
{
	for(int i=0; i<games.size(); i++)
	{
		delete games[i];
	}

	for(int i=0; i<teams.size(); i++)
	{
		delete teams[i];
	}
}

void Tournament::setTeamCount(int num)
{
	if(isValidTeamcount(num))
	{
		teamcnt = num;
		bracketcnt = num/4;
	}
	else throw 1;
}

int Tournament::getTeamCount(void)
{
	return teamcnt;
}

int Tournament::getBracketCount(void)
{
	return bracketcnt;
}

QString Tournament::getTitle(void) const
{
	return title;
}

void Tournament::setTeams(QVector<Team*> &teams)
{
	this->teams = teams;
}

QVector<Team*> Tournament::getTeams(void) const 
{
	return teams;
}

QVector<Game*> Tournament::getGames(void) const
{
	return games;
}

void Tournament::addGame(Game* game) 
{
	games.append(game);
}

void Tournament::addTeam(Team* team)
{
	teams.append(team);
}

void Tournament::setTitle(QString title) 
{
	this->title = title;
}

void Tournament::initTeams(void)
{	
	teams = QVector<Team*>();
	
	for (int i=0; i<teamcnt; i++)
	{	
		addTeam(new Team(i,QString("Team ").append(QString().setNum(i+1)), " ", "")); //"./Resources/flags/Austria.png"));
	}
}

void Tournament::initGames(void)
{
	games = QVector<Game*> ();
	int result[2] = {0,0}; 

	for (int i=0; i<teamcnt-1; i++)
	{
		if(i<teamcnt/2)
			games.insert(i, new Game(teamcnt-2-i, teams[2*i], teams[2*i+1], result, 0, 0));
		else
			games.insert(i, new Game(teamcnt-2-i, new Team(), new Team(), result, games[abs((teamcnt-2-i)*2+4-teamcnt)], games[abs((teamcnt-i-2)*2+3-teamcnt)]));
	}
}

Team::Round Tournament::getMinRound()
{
	return minRound;
}

void Tournament::initPicks(void) {
	if(teams.size()==4) minRound = Team::HALF;
	else if(teams.size()==8) minRound = Team::QUARTER;
	else if(teams.size()==16) minRound = Team::EIGHT;
	else minRound = Team::SIXTEEN;
}

Game* Tournament::getGameByID(int id)
{
	for (int i=0; i<games.size(); i++)
	{
		if(games[i]->getID() == id)
			return games[i];
	}
	return 0;
}

bool Tournament::setPickForTeam(int id, Team::Round round)
{
	bool correct = false;
	Team team;

	for(int i=0; i<teams.size(); i++)
	{
		if(teams[i]->getId()==id) 
		{
			teams[i]->setPick(round);
			correct = true;
		}
	}
	return correct;
}

bool Tournament::isValidTeamcount(const int &val)
{
	return std::binary_search(VALID_TEAMCOUNTS, VALID_TEAMCOUNTS+sizeof(VALID_TEAMCOUNTS)/sizeof(int), val); 
}