#ifndef TOURNAMENT_H
#define TOURNAMENT_H

#include "game.h"
#include <QVector>
#include <QVectorIterator>
#include <QString>
#include <algorithm>


/**
* @class Tournament
*
* @brief Contains informations and handles a Tournament.
*
* Contains everything about the tournament (name, teams,
* games, etc.).
*/ 
class Tournament
{
public:
	Tournament(void);

	/**
	* Creates a new Tournament with "teamcnt"-teams and title.
	* Initializes teams and games with standard values.
	*/
	Tournament(const int teamcnt, QString title);
	~Tournament(void);
	
	/// Possible team counts
	static const int VALID_TEAMCOUNTS[];
	
	//getters
	int getTeamCount(void);
	/**
	* Returns the number of brackets on one side (= number of teams/4)
	*/
	int getBracketCount(void);
	QVector<Team*> getTeams(void) const;
	QVector<Game*> getGames (void) const;
	QString getTitle(void) const;
	Game* getGameByID(int id);
	Team::Round getMinRound(void);

	//setters
	void setTeams(QVector<Team*> &teams);
	void setTeamCount(int num);
	void setTitle(QString title);
	bool setPickForTeam(int id, Team::Round round);

	void addTeam(Team* team);
	void addGame(Game* game);

private:
	QVector<Team*> teams;
	QVector<Game*> games;
	QString title;
	int teamcnt;
	int bracketcnt;
	Team::Round minRound;

	void initTeams(void);
	void initGames(void);
	void initPicks(void);
	bool isValidTeamcount(const int &val);
};

#endif