#include "xmlWriter.h"

XmlWriter::XmlWriter()
{	
	 filename = "default.xml";
}

XmlWriter::XmlWriter(QString filename)
: filename(filename)
{	
}

bool XmlWriter::write (Tournament* tournament)
{	
	bool correct=false;

	QFile file( filename );
	qDebug(filename.toLatin1());
    if( !file.open( IO_ReadWrite  ) )
    {
		qDebug("Loading Failed to load file." );
		return correct;
    }
	
	QDomDocument doc("MyTournament");
    QDomElement root = doc.createElement("tournament");
	root.setAttribute("name", tournament->getTitle());
    doc.appendChild(root);

	//Write teams
    QDomElement tag = doc.createElement("teams");
    root.appendChild(tag);
	QVector<Team*> teams = tournament->getTeams();

	QDomElement temp;
	QDomElement inner;
	QDomText t;
	for(int i=0; i<teams.size(); i++) 
	{
		temp = doc.createElement("team");
		temp.setAttribute("id",QString::number(teams[i]->getId()));

		//name
		inner = doc.createElement("name");
		t = doc.createTextNode(teams[i]->getName());
		inner.appendChild(t);
		temp.appendChild(inner);
		
		//info
		inner = doc.createElement("info");
		t = doc.createTextNode(teams[i]->getInfo());
		inner.appendChild(t);
		temp.appendChild(inner);

		//icon
		inner = doc.createElement("icon");
		t = doc.createTextNode(teams[i]->getIconFileName());
		inner.appendChild(t);
		temp.appendChild(inner);

		//pick
		inner = doc.createElement("pick");
		t = doc.createTextNode(QString::number(teams[i]->getPick()));
		inner.appendChild(t);
		temp.appendChild(inner);
		
		tag.appendChild(temp);
	}

	//write games
	tag = doc.createElement("games");
    root.appendChild(tag);
	QVector<Game*> games = tournament->getGames();

	for(int i=0; i<games.size(); i++) 
	{
		temp = doc.createElement("game");
		temp.setAttribute("id",QString::number(games[i]->getID()));
		
		//team1
		inner = doc.createElement("team1");
		t = doc.createTextNode(QString::number(games[i]->getTeam1()));
		inner.appendChild(t);
		temp.appendChild(inner);

		//team1
		inner = doc.createElement("team2");
		t = doc.createTextNode(QString::number(games[i]->getTeam2()));
		inner.appendChild(t);
		temp.appendChild(inner);

		//result
		inner = doc.createElement("result");
		Game::Result r = games[i]->getResult();
		QString res;
		res.append(QString::number(r.score[0]));
		res.append(":");
		res.append(QString::number(r.score[1]));
		t = doc.createTextNode(res);
		inner.appendChild(t);
		temp.appendChild(inner);

		//date
		inner = doc.createElement("date");
		t = doc.createTextNode(games[i]->getDate());
		inner.appendChild(t);
		temp.appendChild(inner);

		//descr
		inner = doc.createElement("descr");
		t = doc.createTextNode(games[i]->getDescr());
		inner.appendChild(t);
		temp.appendChild(inner);
		
		tag.appendChild(temp);
	}
	

    QString xml = doc.toString();
	file.write("<?xml version='1.0' encoding='utf-8'?>");
	file.write(xml.toLatin1());
 
	file.flush();
	file.close();


	return correct;
}